//---------------------------------------------------------------------------
// ASF decoding
//---------------------------------------------------------------------------

#include <vcl.h>						// ShowMesssage
#pragma hdrstop

#include <io.h>							// filelength
#include <stdio.h>						// fopen

#include "Main.h"						// MainForm
#include "AsfInfo.h"
#include "fileinfo.h"
#include "multinfo.h"

extern struct FileInfo *pai;

typedef __int64 QWORD;				// Also : LONGLONG & struct LARGE_INTEGER

#pragma package(smart_init)

/*
	CLSID	clsid;
	CLSIDFromString (L"{75B22630-668E-11CF-A6D9-00AA0062CE6C}", &clsid);
*/

// Read a string and convert it to a CLSID (GUID)

void getCLSID (char *p, CLSID *clsid)
{
	clsid->Data1 = *(DWORD *)p;
	clsid->Data2 = *(DWORD *)(p+4);
	clsid->Data3 = *(DWORD *)(p+6);
	for (int i=0 ; i<8 ; i++)
		clsid->Data4[i] = *(p+8+i);
}

// Get information for .AS*(ASF) / .WM* (WMF)
// -----------------------------------------------------------

int GetASFinfo (wchar_t *wszFile)
{
	FILE	*fh;
	char	szBuf[MAXHEAD], *p,
    		stream[10];
    int		len,
    		as,							// Audio Stream number
    		sn;							// Stream number (use with stream[])
    DWORD	pos, pos2, headLen, objsize,
			AvgBPS[10],
			dwTotAvgBPS;
	CLSID	CLSID_ASF_Header_Object,
    		CLSID_ASF_Data_Object,
    		CLSID_ASF_File_Properties_Object,
    		CLSID_ASF_Stream_Properties_Object,
    		CLSID_ASF_Header_Extension_Object,
    		CLSID_ASF_Codec_List_Object,
    		CLSID_ASF_Stream_Bitrate_Properties_Object,
            CLSID_ASF_Content_Encryption_Object,
            CLSID_ASF_Extended_Content_Encryption_Object,
    		CLSID_ASF_Extended_Stream_Properties_Object,
    		CLSID_ASF_Audio_Media,
    		CLSID_ASF_Video_Media,
            objid;

	CLSIDFromString (L"{75B22630-668E-11CF-A6D9-00AA0062CE6C}", &CLSID_ASF_Header_Object);
	CLSIDFromString (L"{75B22636-668E-11CF-A6D9-00AA0062CE6C}", &CLSID_ASF_Data_Object);
	CLSIDFromString (L"{8CABDCA1-A947-11CF-8EE4-00C00C205365}", &CLSID_ASF_File_Properties_Object);
	CLSIDFromString (L"{B7DC0791-A9B7-11CF-8EE6-00C00C205365}", &CLSID_ASF_Stream_Properties_Object);
	CLSIDFromString (L"{5FBF03B5-A92E-11CF-8EE3-00C00C205365}", &CLSID_ASF_Header_Extension_Object);
	CLSIDFromString (L"{86D15240-311D-11D0-A3A4-00A0C90348F6}", &CLSID_ASF_Codec_List_Object);
	CLSIDFromString (L"{7BF875CE-468D-11D1-8D82-006097C9A2B2}", &CLSID_ASF_Stream_Bitrate_Properties_Object);
	CLSIDFromString (L"{2211B3FB-BD23-11D2-B4B7-00A0C955FC6E}", &CLSID_ASF_Content_Encryption_Object);
	CLSIDFromString (L"{298AE614-2622-4C17-B935-DAE07EE9289C}", &CLSID_ASF_Extended_Content_Encryption_Object);
	CLSIDFromString (L"{14E6A5CB-C672-4332-8399-A96952065B5A}", &CLSID_ASF_Extended_Stream_Properties_Object);
	CLSIDFromString (L"{F8699E40-5B4D-11CF-A8FD-00805F5C442B}", &CLSID_ASF_Audio_Media);
	CLSIDFromString (L"{BC19EFC0-5B4D-11CF-A8FD-00805F5C442B}", &CLSID_ASF_Video_Media);
	memset(AvgBPS, 0, 10*sizeof(DWORD));
	memset(stream, 0, 10*sizeof(char));

	// Header reading

	fh = _wfopen (wszFile, L"rb");
	if (fh == NULL)
	{	strcpy (szErr, szErrorString[7]);	return -1;	}

	len = fread (szBuf, 1, MAXHEAD, fh);
	if (!len)
	{	strcpy (szErr, szErrorString[7]);	return -1;	}
	fclose (fh);

	// We first read the Header Object structure

	pos = 0;
    p = szBuf;
	getCLSID (p, &objid);
	if (!IsEqualCLSID(objid, CLSID_ASF_Header_Object))
    {	return -1;	}
	headLen = *(QWORD *)(p+16);
	pai->dwFileHeadSize = headLen;
	pos += 30;
	memset (AvgBPS, 0, sizeof (AvgBPS));

    while (pos < headLen)
    {
	    WCHAR	*temp;

		p = szBuf+pos;
		getCLSID (p, &objid);
		StringFromCLSID (objid, &temp);
		objsize = *(QWORD *)(p+16);
        if (!objsize)	// Error in ASF header 
        	break;
		if (IsEqualCLSID(objid, CLSID_ASF_File_Properties_Object))
    	{
			pai->dwFileTheoSize = *(QWORD *)(p+40);
			pai->dwFramesNb = *(QWORD *)(p+56);	// Packets count
			pai->dwFileDuration = *(QWORD *)(p+64)/10000000L;	// Play duration
			pai->dwFileDuration = *(QWORD *)(p+72)/10000000L;	// Send duration
			dwTotAvgBPS = *(DWORD *)(p+100)/8;	// Audio + Video bitrate
        }
		else if (IsEqualCLSID(objid, CLSID_ASF_Header_Extension_Object))
    	{
			len = *(DWORD *)(p+42);
            if (len)					// It works, but unused at this time
            {
//				ShowMessage ("ASF_Header_Extension_Object present");
				getCLSID (p+46, &objid);
				StringFromCLSID (objid, &temp);
			}
        }
		else if (IsEqualCLSID(objid, CLSID_ASF_Stream_Properties_Object))
    	{
			CLSID	streamType;
			getCLSID (p+24, &streamType);
			len = *(QWORD *)(p+56)/10000000L;
			sn = *(WORD *)(p+72) & 0x7F;
			if (IsEqualCLSID(streamType, CLSID_ASF_Audio_Media))
            {
				WAVEFORMATEX	wfx;

				as = pai->wAStreamsNb;
				stream[sn] = as;
                pai->wAStreamsNb++;
				len = sizeof (WAVEFORMATEX);
				len = *(QWORD *)(p+64);
				memcpy (&wfx, p+78, sizeof (WAVEFORMATEX));
				pai->a[as].wAC = wfx.wFormatTag;
				pai->a[as].wAChannels = wfx.nChannels;
				pai->a[as].dwAAvgBPS = wfx.nAvgBytesPerSec;
				pai->a[as].dwASps = wfx.nSamplesPerSec;
            }
			else if (IsEqualCLSID(streamType, CLSID_ASF_Video_Media))
            {
				stream[sn] = 'V';
				pai->dwWidth = *(DWORD *)(p+78);
				pai->dwHeight = *(DWORD *)(p+78+4);
				strncpy (pai->szVC, p+78+11+16, 4);	// FourCC
				pai->szVC[4] = 0;
            }
        }
		else if (IsEqualCLSID(objid, CLSID_ASF_Codec_List_Object))
    	{
			WORD	type;
			DWORD	i, count = *(DWORD *)(p+40);

			pos2 = 44;
			as = 0;
			for (i=0 ; i<count ; i++)
            {
				type = *(WORD *)(p+pos2);
				pos2 += 2;

				// Codec Name

                len = *(WORD *)(p+pos2);
				strcpy (type==1 ? pai->szVCname : pai->a[as++].szACname,
                 	WideCharToString((WCHAR *)(p+pos2+2)).c_str());
				pos2 += 2+2*len;

				// Codec Description (not interesting us)

                len = *(WORD *)(p+pos2);
				pos2 += 2+2*len;

				// Codec information (4CC for video)

                len = *(WORD *)(p+pos2);
				if ((type==1) && (len==4))
                {
					strncpy (pai->szVC, p+pos2+2, 4);
					pai->szVC[4] = 0;
                }
				pos2 += 2+len;
            }
        }
		else if (IsEqualCLSID(objid, CLSID_ASF_Stream_Bitrate_Properties_Object))
    	{
			WORD	count = *(WORD *)(p+24);

			for (int i=0 ; i<count ; i++)
            {
				sn = *(WORD *)(p+26+6*i) & 0x7F;
				AvgBPS[sn] = *(DWORD *)(p+28+6*i) / 8;
            }
        }
		else if (IsEqualCLSID(objid, CLSID_ASF_Content_Encryption_Object))
   		{
//			ShowMessage("Content_Encryption_Object");
        	pai->bDRMlocked = true;
        }
		else if (IsEqualCLSID(objid, CLSID_ASF_Extended_Content_Encryption_Object))
   		{
//			ShowMessage("Extended_Content_Encryption_Object");
        	pai->bDRMlocked = true;
        }
		else if (IsEqualCLSID(objid, CLSID_ASF_Data_Object))
   		{
			break;
        }
        else
        {
//			ShowMessage (temp);
        }
		pos += objsize;
    }

	if (AvgBPS[1])
	{
		for (int i=1 ; i<10 ; i++)
		{
			BYTE	str = stream[i];
			if (!AvgBPS[i] ||  !str)    	break;
			if (str=='V')
					pai->dwVAvgBPS = AvgBPS[i];
			else    pai->a[str].dwAAvgBPS = AvgBPS[i];
        }
    }
	else if (!pai->dwVAvgBPS)
		pai->dwVAvgBPS = dwTotAvgBPS - pai->a[0].dwAAvgBPS;
	return 0;
}
