// Header file for FileInfo.cpp

#ifndef _AVICODEC_FILEINFO_
#define _AVICODEC_FILEINFO_

#include <windows.h>

// AVI std = 0x3000 (but 0X8000 found)
// RM std  = 0x0400
// OGG std = 0x0400 (beginning) but 0x4000 (end if we want to have video stream)
// MPG std =0x20000
#define MAXHEAD 0x4000
#define MAXSAVEDHEADER	0x20000

#define MAXSTREAMS	4					// Max Streams handled
#define MAXAUDIOSTREAMS	3

// Invert DWORD bytes (Little <-> Big endian)
#define INVDW( dw )				\
		(DWORD)( ((dw & 0xFF) << 24) | ((dw & 0xFF00) << 8) |	\
        		 ((dw >> 8) & 0xFF00) | ((dw >> 24) & 0xFF)	)

// Invert WORD bytes
#define INVW( w )				\
		(WORD)( ((w & 0xFF) << 8) | ((w & 0xFF00) >> 8) )

#ifndef min
#define min(a, b)	((a) < (b) ? (a) : (b))
#endif

#define streamtypeDV         mmioFOURCC('i', 'a', 'v', 's')

// Detailled error message

#define SZERR	100
extern char	szErr[SZERR];

enum filetype { ftUNK=-1, ftAVI, ftMPG, ftOGG, ftRMF, ftDSH, ftASF, ftMP3, ftMKV, ftFLV };
enum codecsup { ftUNS=0, ftSUP, ftSNV };	// UNSupported, Supported

// FileInfo structure

struct FileInfo
{
	// GENERAL file info

	filetype	iFileType;					// 0=AVI, 1=MPG, 2=OGG, 3=Real
	char	szFileType[10],
			szFileDate[20],
			szFileInfo[100];
	DWORD	dwFileRealSize,
    		dwFileRealSizeKb,			// For file > 4 Gb size
			dwFileTheoSize,
            dwFileHeadSize,
            dwFileDuration;				// Duration in seconds
    WORD	wFileQuality;				// 0=bad, 100=good
    BOOL	bFileAudio,					// Audio file (only)
    		bDRMlocked;					// Locked with DRM (WMA, WMV)

	// VIDEO stream info

	char	szVC[5],					// Video Codec (= fccHandler)
			szVCname[40],				// VC name
			szVCurl[100],				// VC website
			szVCdriver[30],				// VC driver
            szAspRatio[8];				// Aspect Ratio (text, rounded)
	WORD	wVCsupp;					// 0=Video Codec not supported by this system, 1=supported, 2=not verified
	WORD	wAspRatio;					// Aspect Ratio (exact * 100)
	DWORD	dwWidth;				 	// Width, in pixels
	DWORD	dwHeight;				  	// Height, in pixels
	DWORD	dwFramesNb;					// Nb of frames
	DWORD	dwFps;						// Frames per sec * 1000
	DWORD	dwVSec;						// Duration in seconds
    DWORD   dwVSize;					// Size in bytes
	DWORD	dwVAvgBPS;					// Average Bytes Per Sec

	// AUDIO stream info (2 1st streams if many)

	WORD	wAStreamsNb;				// Nb of Audio streams
    DWORD   dwATotSize;					// Size in bytes
	struct
    {
		WORD	wAC;					// Audio Codec = wFormatTag
		char	szACname[40],			// AC name
				szACurl[100];			// AC website
		WORD	wACsupp;				// Is Audio Codec supported by this system ?

		WORD	wAChannels;				// Channels
		BOOL	bIsCBR;					// CBR / VBR
		DWORD	dwASnb;					// Samples nb
		DWORD	dwASps;					// Samples per sec
		DWORD	dwAAvgBPS;				// Average Bytes Per Sec
		DWORD	dwASec;					// Duration in seconds
    	DWORD   dwASize;				// Size in bytes
    }	a[MAXAUDIOSTREAMS];
};

// Functions

int GetAVIinfo (wchar_t *wszFile);
int GetMPGinfo (wchar_t *wszFile);
int GetOGGinfo (wchar_t *wszFile);
int GetRMinfo (wchar_t *wszFile);
int GetFLVinfo (wchar_t *wszFile);
int GetMediaInfo (wchar_t *wszFile);
int DSHOWtest (wchar_t *szwcFile, BOOL bDetailled);
int GetDSHOWinfo (wchar_t *wszFile);
int FileInfoFill ();
filetype FileType (wchar_t *wszFile, BOOL bSaveHeader);
WORD CalcFQ ();

#endif

