//---------------------------------------------------------------------------

#ifndef MpegInfoH
#define MpegInfoH
//---------------------------------------------------------------------------

#define BUFFER_SIZE		0x30000		// To be able to read last GOP code

// MPG definitions

#define MPEG_SEQUENCE_CODE			0x000001B3
#define MPEG_GOP_CODE				0x000001B8
#define MPEG_PROGRAM_END_CODE		0x000001B9
#define MPEG_PACK_START_CODE		0x000001BA
#define SYSTEM_HEADER_START_CODE	0x000001BB
#define MPEG_PRIVATE_STREAM_1		0x000001BD
#define MPEG_AUDIO_C0_CODE			0x000001C0 // 1st audio stream
#define MPEG_AUDIO_C1_CODE			0x000001C1 // 2nd audio stream id (dual channel)
#define MPEG_AUDIO_C2_CODE			0x000001C2 // 3rd audio stream id (surround sound)
#define MPEG_VIDEO_E0_CODE			0x000001E0
// Start codes 
// const unsigned int PICTURE_START_CODE           = 0x00000100;
// const unsigned int SLICE_START_CODE_MIN         = 0x00000101;
// const unsigned int SLICE_START_CODE_MAX         = 0x000001AF;
// const unsigned int RESERVED_0                   = 0x000001B0;
// const unsigned int RESERVED_1                   = 0x000001B1;
// const unsigned int USER_DATA_START_CODE         = 0x000001B2;
// const unsigned int SEQUENCE_HEADER_CODE         = 0x000001B3;
// const unsigned int SEQUENCE_ERROR_CODE          = 0x000001B4;
// const unsigned int EXTENSION_START_CODE         = 0x000001B5;
// const unsigned int RESERVED_2                   = 0x000001B6;
// const unsigned int SEQUENCE_END_CODE            = 0x000001B7;
// const unsigned int GROUP_START_CODE             = 0x000001B8;
// const unsigned int SYSTEM_START_CODE_MIN        = 0x000001B9;
// const unsigned int SYSTEM_START_CODE_MAX        = 0x000001FF;




#define SUBSTREAM_AC3_0				0x80
#define SUBSTREAM_AC3_8				0x87
#define SUBSTREAM_DTS_0				0x88
#define SUBSTREAM_DTS_8				0x8F
#define SUBSTREAM_PCM_0				0xA0
#define SUBSTREAM_PCM_F				0xAF

#define AUDIO_AC3_SYNC_WORD			0x0B77
#define AUDIO_MPEG_SYNC_WORD		0x0FFF

class MpegInfo
{
private:
	FILE	*fh;
	unsigned char	*bfr;
	int		byteidx,					// Byte index in buffer
    		bitidx,						// Bit index in current byte
            bytecount,					// Bytes already read
			bufcount;					// Bytes currently in buffer
	bool	eobs;						// End of BitStream
	int		masks[8];

	bool __fastcall refill_buffer ();
	unsigned int __fastcall look_ahead(int N);
	unsigned int __fastcall getbits (int N);
	unsigned int __fastcall get1bit ();
public:
	int __fastcall Init (wchar_t *wszFile);
	int __fastcall ParseFile ();
	void __fastcall Close ();
};

// Object class to read bits (and not bytes)

class BitReader
{
private:
	FILE	*fh;
	unsigned char	*bfr;
	int		byteidx,					// Byte index in buffer
    		bitidx,						// Bit index in current byte
            bytecount,					// Bytes already read
			bufcount;					// Bytes currently in buffer
	bool	eobs;						// End of BitStream
	int		masks[8];

	bool __fastcall refill_buffer ();
public:
	__fastcall BitReader (FILE *FileHandle = NULL);
	__fastcall ~BitReader ();
	int __fastcall Open (wchar_t *wszFile);
	void __fastcall Close ();

	DWORD __fastcall look_ahead(int N);
	DWORD __fastcall getbits (int N);
	DWORD __fastcall get1bit ();
};

#endif
