/*----------------------------------------------------------------------------*/
/* DESCRIPTION                                                                */
/*   Parser XML de type pull (include)                                        */
/*                                                                            */
/* HISTORIQUE                                                                 */
/*   Voir XML.cpp                                                             */
/*                                                                            */
/* NOTES                                                                      */
/*   Exemple d'utilisation :                                                  */
/*                                                                            */
/*   XML_CReader *pXMLReader;                                                 */
/*   XML_RESULT xmlResult;                                                    */
/*   char cBuffer[256];                                                       */
/*                                                                            */
/*   pXMLReader = new XML_CReader ("test.xml");                               */
/*   while (true)                                                             */
/*   {                                                                        */
/*     xmlResult = pXMLReader->nextNode ();                                   */
/*     if (xmlResult != XML_OK)                                               */
/*     {                                                                      */
/*       pXMLReader->showResult (xmlResult);                                  */
/*       break;                                                               */
/*     }                                                                      */
/*     pXMLReader->getString (cBuffer);                                       */
/*                                                                            */
/*     if (pXMLReader->isAttribute () == XML_OK)                              */
/*       MessageBox (NULL, cBuffer, "Attribute", MB_OK);                      */
/*                                                                            */
/*     if (pXMLReader->isAttributeValue () == XML_OK)                         */
/*       MessageBox (NULL, cBuffer, "Attribute value", MB_OK);                */
/*                                                                            */
/*     if (pXMLReader->isElementStart () == XML_OK)                           */
/*       MessageBox (NULL, cBuffer, "Element start", MB_OK);                  */
/*                                                                            */
/*     if (pXMLReader->isElementEnd () == XML_OK)                             */
/*       MessageBox (NULL, cBuffer, "Element end", MB_OK);                    */
/*                                                                            */
/*     if (pXMLReader->isElementValue () == XML_OK)                           */
/*       MessageBox (NULL, cBuffer, "Element value", MB_OK);                  */
/*   }                                                                        */
/*   delete (pXMLReader);                                                     */
/*                                                                            */
/* Copyright (c) 2001 www.yragael.com. Tous droits rservs. Reproduction et  */
/* lien interdits sans un accord pralable de l'auteur                        */
/*----------------------------------------------------------------------------*/

#ifndef _INC_XML
#define _INC_XML

/* INCLUDES ------------------------------------------------------------------*/

#include <windows.h>
#include <stdio.h>
#include <string.h>

/* CONSTANTES ----------------------------------------------------------------*/

// Limites pour allocations

#define XML_SIZE_STACK 1024
#define XML_SIZE_BUFFER 1024
#define XML_SIZE_ELEMENTNAME 64

// Codes retourns

#define XML_OK 0
#define XML_NOK 1
#define XML_UNEXPECTEDCHAR 2
#define XML_EOF 3
#define XML_UNEXPECTEDEOF 4
#define XML_BUFFEROVERFLOW 5
#define XML_STACKOVERFLOW 6
#define XML_ELEMENTMISMATCH 7
#define XML_ENDOFELEMENT 8
#define XML_WRONGCONTEXT 9

// Contextes de l'analyse grammaticale

#define XML_CONTEXT_ELEMENT 0
#define XML_CONTEXT_ELEMENTVALUE 1
#define XML_CONTEXT_ATTRIBUTE 2
#define XML_CONTEXT_ATTRIBUTEVALUE 3
#define XML_CONTEXT_ERROR 4
#define XML_CONTEXT_EOF 5

// Types de l'lment courant

#define XML_TYPE_UNDEFINED 0
#define XML_TYPE_ELEMENTSTART 1
#define XML_TYPE_ELEMENTEND 2
#define XML_TYPE_ELEMENTVALUE 3
#define XML_TYPE_ATTRIBUTE 4
#define XML_TYPE_ATTRIBUTEVALUE 5

/* TYPES ---------------------------------------------------------------------*/

typedef int XML_RESULT;

/* CLASSES -------------------------------------------------------------------*/

class XML_CReader
{
  protected :
    FILE *_pFile;
    char *_pcBuffer,
         *_pcStack,
         *_pcStackPointer;
    char _cChar;
    int _iContext,
        _iStackSize,
        _iType,
        _iLine, _iColumn;
    XML_RESULT readNode (void);
    XML_RESULT readElement (void);
    XML_RESULT readElementValue (void);
    XML_RESULT readAttribute (void);
    XML_RESULT readAttributeValue (void);

  public :
    XML_CReader (char *);
    ~XML_CReader (void);
    XML_RESULT isElementStart (void);
    XML_RESULT isElementEnd (void);
    XML_RESULT isElementValue (void);
    XML_RESULT isAttribute (void);
    XML_RESULT isAttributeValue (void);
    XML_RESULT getString (char *);
    XML_RESULT nextNode (void);
    XML_RESULT nextElement (void);
    XML_RESULT nextElementValue (void);
    XML_RESULT nextAttribute (void);
    XML_RESULT nextAttributeValue (void);
    XML_RESULT getPosition (int *, int *);
    XML_RESULT showResult (XML_RESULT);
};

#endif